package web

import (
	"context"

	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol/json"
	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	"git.loafle.net/overflow/overflow_gateway_app/external/grpc"
	"google.golang.org/grpc/metadata"
)

func newRCPServerHandler() gateway.ServerHandler {
	sh := &RPCServerHandlers{}
	sh.RegisterCodec(json.NewServerCodec(), "json")
	return sh
}

type RPCServerHandlers struct {
	gateway.ServerHandlers
}

func (rpcSH *RPCServerHandlers) Invoke(ctx context.Context, method string, params []string) (result interface{}, err error) {

	soc := ctx.Value("socket").(*cwf.Socket)

	md := metadata.Pairs("email", soc.ID().(string))
	grpcCTX := metadata.NewOutgoingContext(context.Background(), md)

	r, err := grpc.Exec(grpcCTX, method, params)
	if err != nil {
		return nil, err
	}

	return r, nil
}

func (rpcSH *RPCServerHandlers) Validate() {
	rpcSH.ServerHandlers.Validate()
}

type Result struct {
	Name  string `json:"name,omitempty"`
	Count int    `json:"count,omitempty"`
}
