package web

import (
	"context"
	"log"

	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/server"
	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	"github.com/valyala/fasthttp"
)

func NewSocketHandler() cwf.SocketHandler {
	sh := &SocketHandlers{}
	sh.rpcServerHandler = newRCPServerHandler()

	return sh
}

type SocketHandlers struct {
	cwf.SocketHandlers

	rpcServerHandler gateway.ServerHandler
}

func (sh *SocketHandlers) Handshake(ctx *fasthttp.RequestCtx) (interface{}, *fasthttp.ResponseHeader) {
	return "overflow@loafle.com", nil
}

func (sh *SocketHandlers) Handle(soc *cwf.Socket, stopChan <-chan struct{}, doneChan chan<- struct{}) {
	contentType := "json"
	codec, err := sh.rpcServerHandler.GetCodec(contentType)
	if nil != err {
		log.Printf("RPC Handle: %v", err)
		doneChan <- struct{}{}
		return
	}

	var socConn *cwf.SocketConn
	baseCtx := context.Background()
	// conn.SetPongHandler(func(string) error { c.conn.SetReadDeadline(time.Now().Add(pongWait)); return nil })

	for {
		if socConn, err = soc.WaitRequest(); nil != err {
			doneChan <- struct{}{}
			return
		}

		// // "git.loafle.net/commons_go/websocket_fasthttp/websocket"
		// switch socConn.MessageType {
		// case websocket.TextMessage:
		// case websocket.BinaryMessage:
		// }
		ctx := context.WithValue(baseCtx, "socket", soc)

		if err = gateway.Handle(ctx, sh.rpcServerHandler, codec, socConn, socConn); nil != err {
			if server.IsClientDisconnect(err) {
				doneChan <- struct{}{}
				return
			}
			log.Printf("RPC: %v", err)
		}

		if err = socConn.Close(); nil != err {
			doneChan <- struct{}{}
			return
		}

		select {
		case <-stopChan:
			return
		default:
		}
	}

}

func (sh *SocketHandlers) Validate() {
	sh.SocketHandlers.Validate()

	if nil == sh.rpcServerHandler {
		panic("RPCServerHandler must be specified.")
	}
}
